@@ -65,8 +65,6 @@ sub parse {
   
   $data =~ s/\r\n?/\n/gs ;
   
-  $data =~ s/^\s*<\?xml.*?>//gsi ;
-  
   my @parsed ;
   
   while( $data =~ /(.*?)<(.*?)>/gsi ) {
@@ -79,64 +77,48 @@ sub parse {
       my $more ;
       ($more) = ( $data =~ /\G(.*?)>/s ) ;
       pos($data) += length($more) + 1 ;
-      $markup = $markup.'>'.$more ;
-      ($more_q , @args) = &parse_tag($markup) ;
+      ($more_q , @args) = &parse_tag($markup.'>'.$more) ;
     }
     
     if ($cont =~ /\S/s) { push(@parsed , 'Char' , $cont) ;}
     
     if ($args[0] =~ /^\/(.*)/) { push(@parsed , 'End' , $1) ;}
-    elsif (@args[-1] =~ /^\/$/) {
-      pop @args ;
-      push(@parsed , 'StartEnd' , [@args]) ;
-    }
     else { push(@parsed , 'Start' , [@args]) ;}
   }
   
   {
-  
-    my (%close,@close,%open) ;
+    my (@open,%close) ;
     for(my $i=$#parsed-1 ; $i >= 0 ; $i-=2) {
       my $type = $parsed[$i] ;
       
       if ($type eq 'End') {
         my $tag = $parsed[$i+1] ;
-        $close{lc($tag)}++ ;
-        push(@close , $i) ;
+        $close{$tag}++ ;
       }
       elsif ($type eq 'Start') {
         my $tag = @{$parsed[$i+1]}[0] ;
         
-        if (!$close{lc($tag)}) {
+        if (!$close{$tag}) {
           if (@{$parsed[$i+1]}[-1] eq '/' && $#{$parsed[$i+1]} % 2 ) {
             pop @{$parsed[$i+1]} ;
             $parsed[$i] = 'StartEnd' ;
           }
           elsif ($parsed[$i+2] ne 'Char') { $parsed[$i] = 'StartEnd' ;}
-          else {
-            push( @{ $open{@close[-1]} }  , 'End' , $tag) ;
-          }
+          else { push(@open , $tag) ;}
         }
         else {
-          $close{lc($tag)}-- ;
-          pop(@close) ;
+          $close{$tag}-- ;
         }
       }
     }
     
-    if ( %open ) {
-      my @parsed2 ;
-      for(my $i=0 ; $i <= $#parsed ; ++$i) {
-        push(@parsed2 , @{$open{$i}}) if $open{$i} ;
-        push(@parsed2 , $parsed[$i]) ;
-      }
-      @parsed = @parsed2 ;
+    foreach my $open_i ( @open ) {
+      push(@parsed , 'End' , $open_i ) ;
     }
-
   }
 
   &{$this->{Init}}($this) ;
-  
+
   for (my $i = 0 ; $i <= $#parsed ; $i+=2) {
     my $type = $parsed[$i] ;
     my $args = $parsed[$i+1] ;
@@ -238,6 +220,67 @@ sub parse_tag {
 # END #
 #######
 
+package main ;
+
+if (0) {
+  my $parser = XML::Smart::HTMLParser->new();
+  
+  $parser->parse('<root><foo bar="x"/></root>') ; exit ;
+  
+  $parser->parse(q`
+  <foo1 baz="y1=name\" bar1=x1 > end" w=q>
+  data1
+  <foo2 bar2="" arg0 x=y>
+  data2
+  </foo2>
+  <foo3 bar3=x3>
+  `);
+}
 1;
 
+__END__
+
+=head1 NAME
+
+MOD - title
+
+=head1 DESCRIPTION
+
+Desc
+
+=head1 USAGE
+
+  use MOD ;
+
+  ...
+
+
+=head1 METHODS
+
+=over 10
+
+=item Safe
+
+=back
+
+=head2 methodx
+
+=head2 methody
+
+=head1 SEE ALSO
+
+L<MOD>, L<MOD::SUB>,
+
+=head1 AUTHOR
+
+Graciliano M. P. <gm@virtuasites.com.br>
+
+I will appreciate any type of feedback (include your opinions and/or suggestions). ;-P
+
+=head1 COPYRIGHT
+
+This program is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
 
